package com.bn.ark.one.access.provider.core;

package ..deTemp.1731915214418;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bn.ark.enterprise.common.model.Department;
import com.bn.ark.enterprise.common.model.Enterprise;
import com.bn.ark.enterprise.common.model.Staff;
import com.bn.ark.one.access.provider.core.event.DepartmentChangeEvent;
import com.bn.ark.one.access.provider.core.event.EnterpriseChangeEvent;
import com.bn.ark.one.access.provider.core.event.UserChangeEvent;
import com.bn.ark.one.access.provider.core.feign.EnterpriseFeign;
import com.bn.ark.one.access.provider.core.feign.UserEngineFeign;
import com.bn.ark.one.access.provider.core.lang.DepartmentAction;
import com.bn.ark.one.access.provider.core.lang.EnterpriseAction;
import com.bn.ark.one.access.provider.core.lang.UserAction;
import com.bn.ark.one.access.provider.lang.OneAccessProviderProperties;
import com.bn.ark.one.access.provider.model.OriginAccount;
import com.bn.ark.one.access.provider.model.OriginOrgan;
import com.bridata.common.toolkit.StringPlusUtils;
import com.bridata.spring.adapter.EventMulticastAdapter;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.Objects;
import java.util.Optional;
import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;
import org.redisson.api.RMap;
import org.redisson.api.RedissonClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEvent;
import org.springframework.stereotype.Service;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

@Service
public class OneAccessDataService {
  private static final Logger log = LoggerFactory.getLogger(com.bn.ark.one.access.provider.core.OneAccessDataService.class);
  
  private static final String OPERATE_TYPE_ADD = "add";
  
  private static final String OPERATE_TYPE_UPDATE = "update";
  
  private static final String OPERATE_TYPE_DELETE = "delete";
  
  private static Charset CHARSET_UTF_8 = StandardCharsets.UTF_8;
  
  private static final String ALGORITHM = "AES/ECB/PKCS5Padding";
  
  @Autowired
  private EnterpriseFeign enterpriseFeign;
  
  @Autowired
  private UserEngineFeign userEngineFeign;
  
  @Autowired
  private OneAccessProviderProperties properties;
  
  @Autowired
  private RedissonClient redissonClient;
  
  public void handleAccount(OriginAccount originAccount) {
    switch (originAccount.getOperateType()) {
      case "add":
        add(originAccount);
        break;
      case "update":
        update(originAccount);
        break;
      case "delete":
        delete(originAccount);
        break;
    } 
  }
  
  public void handleOrgan(OriginOrgan originOrgan) {
    switch (originOrgan.getOperateType()) {
      case "add":
        add(originOrgan);
        break;
      case "update":
        update(originOrgan);
        break;
      case "delete":
        delete(originOrgan);
        break;
    } 
  }
  
  private void add(OriginAccount originAccount) {
    Enterprise enterpriseByOther = getEnterpriseByOther(originAccount.getOrganizationId());
    if (Objects.nonNull(enterpriseByOther)) {
      EventMulticastAdapter.publishEvent((ApplicationEvent)new UserChangeEvent(originAccount, enterpriseByOther, null, UserAction.ADD));
    } else {
      Department deptByOther = getDeptByOther(originAccount.getOrganizationId());
      EventMulticastAdapter.publishEvent((ApplicationEvent)new UserChangeEvent(originAccount, deptByOther, null, UserAction.ADD));
    } 
  }
  
  private void add(OriginOrgan originOrgan) {
    if (StringPlusUtils.isBlank(originOrgan.getParentId())) {
      EventMulticastAdapter.publishEvent((ApplicationEvent)new EnterpriseChangeEvent(originOrgan, null, EnterpriseAction.ADD));
    } else {
      Department deptByOther = getDeptByOther(originOrgan.getParentId());
      if (Objects.nonNull(deptByOther)) {
        EventMulticastAdapter.publishEvent((ApplicationEvent)new DepartmentChangeEvent(originOrgan, deptByOther, null, DepartmentAction.ADD));
      } else {
        Enterprise enterpriseByOther = getEnterpriseByOther(originOrgan.getParentId());
        if (Objects.nonNull(enterpriseByOther))
          EventMulticastAdapter.publishEvent((ApplicationEvent)new DepartmentChangeEvent(originOrgan, enterpriseByOther, null, DepartmentAction.ADD)); 
      } 
    } 
  }
  
  private void update(OriginAccount originAccount) {
    Optional.<Staff>ofNullable(getStaffByOther(originAccount.getId())).ifPresent(record -> {
          log.info("originAccount的值:" + JSONObject.toJSONString(originAccount));
          log.info("修改的用户信息" + JSONObject.toJSONString(record));
          if (StringPlusUtils.isBlank(originAccount.getOrganizationId())) {
            EventMulticastAdapter.publishEvent((ApplicationEvent)new UserChangeEvent(originAccount, null, record.getStaffId(), UserAction.MODIFY));
          } else {
            Department department = Optional.<String>ofNullable(originAccount.getOrganizationId()).map(()).orElse(null);
            if (department == null) {
              Enterprise enterprise = Optional.<String>ofNullable(originAccount.getOrganizationId()).map(()).orElse(null);
              EventMulticastAdapter.publishEvent((ApplicationEvent)new UserChangeEvent(originAccount, enterprise, record.getStaffId(), UserAction.MODIFY));
            } else {
              EventMulticastAdapter.publishEvent((ApplicationEvent)new UserChangeEvent(originAccount, department, record.getStaffId(), UserAction.MODIFY));
            } 
          } 
        });
  }
  
  private void update(OriginOrgan originOrgan) {
    Enterprise enterprise = getEnterpriseByOther(originOrgan.getId());
    log.info("originOrgan的值:" + JSONObject.toJSONString(originOrgan));
    if (Objects.nonNull(enterprise)) {
      log.info("修改企业信息： {}", JSON.toJSONString(enterprise));
      EventMulticastAdapter.publishEvent((ApplicationEvent)new EnterpriseChangeEvent(originOrgan, enterprise.getEnterpriseId(), EnterpriseAction.MODIFY));
    } else {
      Department department = getDeptByOther(originOrgan.getId());
      if (Objects.nonNull(department)) {
        log.info("修改部门信息： {}", JSON.toJSONString(department));
        if (StringPlusUtils.isBlank(originOrgan.getParentId())) {
          EventMulticastAdapter.publishEvent((ApplicationEvent)new DepartmentChangeEvent(originOrgan, null, department.getOrganId(), DepartmentAction.MODIFY));
        } else {
          Department departByOther = getDeptByOther(originOrgan.getParentId());
          if (departByOther != null) {
            EventMulticastAdapter.publishEvent((ApplicationEvent)new DepartmentChangeEvent(originOrgan, departByOther, department.getOrganId(), DepartmentAction.MODIFY));
          } else {
            Enterprise enterpriseByOther = getEnterpriseByOther(originOrgan.getParentId());
            if (enterpriseByOther != null)
              EventMulticastAdapter.publishEvent((ApplicationEvent)new DepartmentChangeEvent(originOrgan, enterpriseByOther, department.getOrganId(), DepartmentAction.MODIFY)); 
          } 
        } 
      } else {
        throw new RuntimeException("未查询到相关机构ID");
      } 
    } 
  }
  
  private void delete(OriginAccount originAccount) {
    EventMulticastAdapter.publishEvent((ApplicationEvent)new UserChangeEvent(originAccount, null, null, UserAction.DELETE));
  }
  
  private void delete(OriginOrgan originOrgan) {
    Enterprise enterprise = getEnterpriseByOther(originOrgan.getId());
    if (Objects.nonNull(enterprise)) {
      EventMulticastAdapter.publishEvent((ApplicationEvent)new EnterpriseChangeEvent(originOrgan, enterprise.getEnterpriseId(), EnterpriseAction.DELETE));
    } else {
      Optional.<Department>ofNullable(getDeptByOther(originOrgan.getId())).ifPresent(dept -> EventMulticastAdapter.publishEvent((ApplicationEvent)new DepartmentChangeEvent(originOrgan, null, dept.getOrganId(), DepartmentAction.DELETE)));
    } 
  }
  
  public String encrypt(String content, SecretKey secretKey) throws InvalidKeyException, NoSuchAlgorithmException, NoSuchPaddingException, IllegalBlockSizeException, BadPaddingException {
    byte[] result = aes(content.getBytes(CHARSET_UTF_8), 1, secretKey);
    BASE64Encoder base64Encoder = new BASE64Encoder();
    return base64Encoder.encode(result);
  }
  
  public String decrypt(String content, SecretKey secretKey) {
    try {
      BASE64Decoder base64Decoder = new BASE64Decoder();
      byte[] result = aes(base64Decoder.decodeBuffer(content), 2, secretKey);
      return new String(result, CHARSET_UTF_8);
    } catch (Exception e) {
      log.info("解密错误：{}", e.getMessage());
      return null;
    } 
  }
  
  private byte[] aes(byte[] contentArray, int mode, SecretKey secretKey) throws NoSuchAlgorithmException, NoSuchPaddingException, InvalidKeyException, IllegalBlockSizeException, BadPaddingException {
    Cipher cipher = Cipher.getInstance("AES/ECB/PKCS5Padding");
    cipher.init(mode, secretKey);
    byte[] result = cipher.doFinal(contentArray);
    return result;
  }
  
  public SecretKey generateKey(String key) {
    byte[] raw = key.getBytes(CHARSET_UTF_8);
    SecretKeySpec skeySpec = new SecretKeySpec(raw, "AES");
    return skeySpec;
  }
  
  private Enterprise getEnterpriseByOther(String id) {
    return Optional.<RMap>ofNullable(this.redissonClient.getMap("com.bn.ark.enterprise.engine.cache.EnterpriseCache")).map(records -> {
          if (records.values().stream().anyMatch(())) {
            Object res = records.values().stream().filter(()).findAny().get();
            return (Enterprise)JSONObject.parseObject(JSONObject.toJSONString(res), Enterprise.class);
          } 
          return null;
        }).orElse(null);
  }
  
  private Department getDeptByOther(String id) {
    return Optional.<RMap>ofNullable(this.redissonClient.getMap("com.bn.ark.enterprise.engine.cache.DepartmentCache")).map(records -> {
          if (records.values().stream().anyMatch(())) {
            Object res = records.values().stream().filter(()).findAny().get();
            return (Department)JSONObject.parseObject(JSONObject.toJSONString(res), Department.class);
          } 
          return null;
        }).orElse(null);
  }
  
  private Staff getStaffByOther(String id) {
    return Optional.<RMap>ofNullable(this.redissonClient.getMap("com.bn.ark.enterprise.engine.cache.StaffCache")).map(records -> {
          if (records.values().stream().anyMatch(())) {
            Object res = records.values().stream().filter(()).findAny().get();
            return (Staff)JSONObject.parseObject(JSONObject.toJSONString(res), Staff.class);
          } 
          return null;
        }).orElse(null);
  }
}
